@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.4.1.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 #ifndef YY_YY_UTIL_CONFIGPARSER_H_INCLUDED
 # define YY_YY_UTIL_CONFIGPARSER_H_INCLUDED
 /* Debug traces.  */
@@ -250,68 +254,76 @@ extern int yydebug;
     VAR_SERVE_EXPIRED = 460,
     VAR_SERVE_EXPIRED_TTL = 461,
     VAR_SERVE_EXPIRED_TTL_RESET = 462,
-    VAR_FAKE_DSA = 463,
-    VAR_FAKE_SHA1 = 464,
-    VAR_LOG_IDENTITY = 465,
-    VAR_HIDE_TRUSTANCHOR = 466,
-    VAR_TRUST_ANCHOR_SIGNALING = 467,
-    VAR_AGGRESSIVE_NSEC = 468,
-    VAR_USE_SYSTEMD = 469,
-    VAR_SHM_ENABLE = 470,
-    VAR_SHM_KEY = 471,
-    VAR_ROOT_KEY_SENTINEL = 472,
-    VAR_DNSCRYPT = 473,
-    VAR_DNSCRYPT_ENABLE = 474,
-    VAR_DNSCRYPT_PORT = 475,
-    VAR_DNSCRYPT_PROVIDER = 476,
-    VAR_DNSCRYPT_SECRET_KEY = 477,
-    VAR_DNSCRYPT_PROVIDER_CERT = 478,
-    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 479,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 480,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 481,
-    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 482,
-    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 483,
-    VAR_IPSECMOD_ENABLED = 484,
-    VAR_IPSECMOD_HOOK = 485,
-    VAR_IPSECMOD_IGNORE_BOGUS = 486,
-    VAR_IPSECMOD_MAX_TTL = 487,
-    VAR_IPSECMOD_WHITELIST = 488,
-    VAR_IPSECMOD_STRICT = 489,
-    VAR_CACHEDB = 490,
-    VAR_CACHEDB_BACKEND = 491,
-    VAR_CACHEDB_SECRETSEED = 492,
-    VAR_CACHEDB_REDISHOST = 493,
-    VAR_CACHEDB_REDISPORT = 494,
-    VAR_CACHEDB_REDISTIMEOUT = 495,
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 496,
-    VAR_FOR_UPSTREAM = 497,
-    VAR_AUTH_ZONE = 498,
-    VAR_ZONEFILE = 499,
-    VAR_MASTER = 500,
-    VAR_URL = 501,
-    VAR_FOR_DOWNSTREAM = 502,
-    VAR_FALLBACK_ENABLED = 503,
-    VAR_TLS_ADDITIONAL_PORT = 504,
-    VAR_LOW_RTT = 505,
-    VAR_LOW_RTT_PERMIL = 506,
-    VAR_FAST_SERVER_PERMIL = 507,
-    VAR_FAST_SERVER_NUM = 508,
-    VAR_ALLOW_NOTIFY = 509,
-    VAR_TLS_WIN_CERT = 510,
-    VAR_TCP_CONNECTION_LIMIT = 511,
-    VAR_FORWARD_NO_CACHE = 512,
-    VAR_STUB_NO_CACHE = 513,
-    VAR_LOG_SERVFAIL = 514,
-    VAR_DENY_ANY = 515,
-    VAR_UNKNOWN_SERVER_TIME_LIMIT = 516,
-    VAR_LOG_TAG_QUERYREPLY = 517,
-    VAR_STREAM_WAIT_SIZE = 518,
-    VAR_TLS_CIPHERS = 519,
-    VAR_TLS_CIPHERSUITES = 520,
-    VAR_TLS_SESSION_TICKET_KEYS = 521,
-    VAR_IPSET = 522,
-    VAR_IPSET_NAME_V4 = 523,
-    VAR_IPSET_NAME_V6 = 524
+    VAR_SERVE_EXPIRED_REPLY_TTL = 463,
+    VAR_SERVE_EXPIRED_CLIENT_TIMEOUT = 464,
+    VAR_FAKE_DSA = 465,
+    VAR_FAKE_SHA1 = 466,
+    VAR_LOG_IDENTITY = 467,
+    VAR_HIDE_TRUSTANCHOR = 468,
+    VAR_TRUST_ANCHOR_SIGNALING = 469,
+    VAR_AGGRESSIVE_NSEC = 470,
+    VAR_USE_SYSTEMD = 471,
+    VAR_SHM_ENABLE = 472,
+    VAR_SHM_KEY = 473,
+    VAR_ROOT_KEY_SENTINEL = 474,
+    VAR_DNSCRYPT = 475,
+    VAR_DNSCRYPT_ENABLE = 476,
+    VAR_DNSCRYPT_PORT = 477,
+    VAR_DNSCRYPT_PROVIDER = 478,
+    VAR_DNSCRYPT_SECRET_KEY = 479,
+    VAR_DNSCRYPT_PROVIDER_CERT = 480,
+    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 481,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 482,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 483,
+    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 484,
+    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 485,
+    VAR_IPSECMOD_ENABLED = 486,
+    VAR_IPSECMOD_HOOK = 487,
+    VAR_IPSECMOD_IGNORE_BOGUS = 488,
+    VAR_IPSECMOD_MAX_TTL = 489,
+    VAR_IPSECMOD_WHITELIST = 490,
+    VAR_IPSECMOD_STRICT = 491,
+    VAR_CACHEDB = 492,
+    VAR_CACHEDB_BACKEND = 493,
+    VAR_CACHEDB_SECRETSEED = 494,
+    VAR_CACHEDB_REDISHOST = 495,
+    VAR_CACHEDB_REDISPORT = 496,
+    VAR_CACHEDB_REDISTIMEOUT = 497,
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 498,
+    VAR_FOR_UPSTREAM = 499,
+    VAR_AUTH_ZONE = 500,
+    VAR_ZONEFILE = 501,
+    VAR_MASTER = 502,
+    VAR_URL = 503,
+    VAR_FOR_DOWNSTREAM = 504,
+    VAR_FALLBACK_ENABLED = 505,
+    VAR_TLS_ADDITIONAL_PORT = 506,
+    VAR_LOW_RTT = 507,
+    VAR_LOW_RTT_PERMIL = 508,
+    VAR_FAST_SERVER_PERMIL = 509,
+    VAR_FAST_SERVER_NUM = 510,
+    VAR_ALLOW_NOTIFY = 511,
+    VAR_TLS_WIN_CERT = 512,
+    VAR_TCP_CONNECTION_LIMIT = 513,
+    VAR_FORWARD_NO_CACHE = 514,
+    VAR_STUB_NO_CACHE = 515,
+    VAR_LOG_SERVFAIL = 516,
+    VAR_DENY_ANY = 517,
+    VAR_UNKNOWN_SERVER_TIME_LIMIT = 518,
+    VAR_LOG_TAG_QUERYREPLY = 519,
+    VAR_STREAM_WAIT_SIZE = 520,
+    VAR_TLS_CIPHERS = 521,
+    VAR_TLS_CIPHERSUITES = 522,
+    VAR_IPSET = 523,
+    VAR_IPSET_NAME_V4 = 524,
+    VAR_IPSET_NAME_V6 = 525,
+    VAR_TLS_SESSION_TICKET_KEYS = 526,
+    VAR_RPZ = 527,
+    VAR_TAGS = 528,
+    VAR_RPZ_ACTION_OVERRIDE = 529,
+    VAR_RPZ_CNAME_OVERRIDE = 530,
+    VAR_RPZ_LOG = 531,
+    VAR_RPZ_LOG_NAME = 532
   };
 #endif
 /* Tokens.  */
@@ -520,81 +532,88 @@ extern int yydebug;
 #define VAR_SERVE_EXPIRED 460
 #define VAR_SERVE_EXPIRED_TTL 461
 #define VAR_SERVE_EXPIRED_TTL_RESET 462
-#define VAR_FAKE_DSA 463
-#define VAR_FAKE_SHA1 464
-#define VAR_LOG_IDENTITY 465
-#define VAR_HIDE_TRUSTANCHOR 466
-#define VAR_TRUST_ANCHOR_SIGNALING 467
-#define VAR_AGGRESSIVE_NSEC 468
-#define VAR_USE_SYSTEMD 469
-#define VAR_SHM_ENABLE 470
-#define VAR_SHM_KEY 471
-#define VAR_ROOT_KEY_SENTINEL 472
-#define VAR_DNSCRYPT 473
-#define VAR_DNSCRYPT_ENABLE 474
-#define VAR_DNSCRYPT_PORT 475
-#define VAR_DNSCRYPT_PROVIDER 476
-#define VAR_DNSCRYPT_SECRET_KEY 477
-#define VAR_DNSCRYPT_PROVIDER_CERT 478
-#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 479
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 480
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 481
-#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 482
-#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 483
-#define VAR_IPSECMOD_ENABLED 484
-#define VAR_IPSECMOD_HOOK 485
-#define VAR_IPSECMOD_IGNORE_BOGUS 486
-#define VAR_IPSECMOD_MAX_TTL 487
-#define VAR_IPSECMOD_WHITELIST 488
-#define VAR_IPSECMOD_STRICT 489
-#define VAR_CACHEDB 490
-#define VAR_CACHEDB_BACKEND 491
-#define VAR_CACHEDB_SECRETSEED 492
-#define VAR_CACHEDB_REDISHOST 493
-#define VAR_CACHEDB_REDISPORT 494
-#define VAR_CACHEDB_REDISTIMEOUT 495
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 496
-#define VAR_FOR_UPSTREAM 497
-#define VAR_AUTH_ZONE 498
-#define VAR_ZONEFILE 499
-#define VAR_MASTER 500
-#define VAR_URL 501
-#define VAR_FOR_DOWNSTREAM 502
-#define VAR_FALLBACK_ENABLED 503
-#define VAR_TLS_ADDITIONAL_PORT 504
-#define VAR_LOW_RTT 505
-#define VAR_LOW_RTT_PERMIL 506
-#define VAR_FAST_SERVER_PERMIL 507
-#define VAR_FAST_SERVER_NUM 508
-#define VAR_ALLOW_NOTIFY 509
-#define VAR_TLS_WIN_CERT 510
-#define VAR_TCP_CONNECTION_LIMIT 511
-#define VAR_FORWARD_NO_CACHE 512
-#define VAR_STUB_NO_CACHE 513
-#define VAR_LOG_SERVFAIL 514
-#define VAR_DENY_ANY 515
-#define VAR_UNKNOWN_SERVER_TIME_LIMIT 516
-#define VAR_LOG_TAG_QUERYREPLY 517
-#define VAR_STREAM_WAIT_SIZE 518
-#define VAR_TLS_CIPHERS 519
-#define VAR_TLS_CIPHERSUITES 520
-#define VAR_TLS_SESSION_TICKET_KEYS 521
-#define VAR_IPSET 522
-#define VAR_IPSET_NAME_V4 523
-#define VAR_IPSET_NAME_V6 524
+#define VAR_SERVE_EXPIRED_REPLY_TTL 463
+#define VAR_SERVE_EXPIRED_CLIENT_TIMEOUT 464
+#define VAR_FAKE_DSA 465
+#define VAR_FAKE_SHA1 466
+#define VAR_LOG_IDENTITY 467
+#define VAR_HIDE_TRUSTANCHOR 468
+#define VAR_TRUST_ANCHOR_SIGNALING 469
+#define VAR_AGGRESSIVE_NSEC 470
+#define VAR_USE_SYSTEMD 471
+#define VAR_SHM_ENABLE 472
+#define VAR_SHM_KEY 473
+#define VAR_ROOT_KEY_SENTINEL 474
+#define VAR_DNSCRYPT 475
+#define VAR_DNSCRYPT_ENABLE 476
+#define VAR_DNSCRYPT_PORT 477
+#define VAR_DNSCRYPT_PROVIDER 478
+#define VAR_DNSCRYPT_SECRET_KEY 479
+#define VAR_DNSCRYPT_PROVIDER_CERT 480
+#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 481
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 482
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 483
+#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 484
+#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 485
+#define VAR_IPSECMOD_ENABLED 486
+#define VAR_IPSECMOD_HOOK 487
+#define VAR_IPSECMOD_IGNORE_BOGUS 488
+#define VAR_IPSECMOD_MAX_TTL 489
+#define VAR_IPSECMOD_WHITELIST 490
+#define VAR_IPSECMOD_STRICT 491
+#define VAR_CACHEDB 492
+#define VAR_CACHEDB_BACKEND 493
+#define VAR_CACHEDB_SECRETSEED 494
+#define VAR_CACHEDB_REDISHOST 495
+#define VAR_CACHEDB_REDISPORT 496
+#define VAR_CACHEDB_REDISTIMEOUT 497
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 498
+#define VAR_FOR_UPSTREAM 499
+#define VAR_AUTH_ZONE 500
+#define VAR_ZONEFILE 501
+#define VAR_MASTER 502
+#define VAR_URL 503
+#define VAR_FOR_DOWNSTREAM 504
+#define VAR_FALLBACK_ENABLED 505
+#define VAR_TLS_ADDITIONAL_PORT 506
+#define VAR_LOW_RTT 507
+#define VAR_LOW_RTT_PERMIL 508
+#define VAR_FAST_SERVER_PERMIL 509
+#define VAR_FAST_SERVER_NUM 510
+#define VAR_ALLOW_NOTIFY 511
+#define VAR_TLS_WIN_CERT 512
+#define VAR_TCP_CONNECTION_LIMIT 513
+#define VAR_FORWARD_NO_CACHE 514
+#define VAR_STUB_NO_CACHE 515
+#define VAR_LOG_SERVFAIL 516
+#define VAR_DENY_ANY 517
+#define VAR_UNKNOWN_SERVER_TIME_LIMIT 518
+#define VAR_LOG_TAG_QUERYREPLY 519
+#define VAR_STREAM_WAIT_SIZE 520
+#define VAR_TLS_CIPHERS 521
+#define VAR_TLS_CIPHERSUITES 522
+#define VAR_IPSET 523
+#define VAR_IPSET_NAME_V4 524
+#define VAR_IPSET_NAME_V6 525
+#define VAR_TLS_SESSION_TICKET_KEYS 526
+#define VAR_RPZ 527
+#define VAR_TAGS 528
+#define VAR_RPZ_ACTION_OVERRIDE 529
+#define VAR_RPZ_CNAME_OVERRIDE 530
+#define VAR_RPZ_LOG 531
+#define VAR_RPZ_LOG_NAME 532
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-
 union YYSTYPE
 {
-#line 66 "util/configparser.y" /* yacc.c:1909  */
+#line 66 "util/configparser.y"
 
 	char*	str;
 
-#line 596 "util/configparser.h" /* yacc.c:1909  */
-};
+#line 615 "util/configparser.h"
 
+};
 typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
